// PIC Source File of USB Logic Analyzer & Generator
// Designed by T.Iwata at DIGITALFILTER.COM 2006/09/14

#include "UsbLogPic.h"
#include <stdlib.h>

#define bkbhit (next_in!=next_out)
#define BUFFER_SIZE 48

char RX_Buff[BUFFER_SIZE];
char next_in = 0;
char next_out = 0;
char cr_flag = 0;


void  Init_IO(void)
{
	setup_adc_ports(NO_ANALOGS);

	#use fast_io(A)
	#use fast_io(E)
	#use fast_io(B)
	#use fast_io(C)

	set_tris_A(0x2d); // A5, A3, A2 and A0 are inputs, A4 and A1 are outputs
	set_tris_E(0x01); // E0 is input 
	set_tris_C(0x00); // Port C is output
	set_tris_B(0xff); // Port B is input

	port_b_pullups(TRUE);

	setup_timer_1(T1_DISABLED);

	output_low(PIN_C0);
	delay_ms(1);
	output_high(PIN_C0); // RST_N
	output_low(PIN_C1); // CAPTURE
	output_low(PIN_C2); // DOWNLOAD
	output_low(PIN_C3); // PACKET
	output_low(PIN_C4); // STROBE
	output_low(PIN_C5); // REGLATCH
	output_low(PIN_C6); // REGCLK
	output_low(PIN_C7); // REGDATA
	output_low(PIN_A1); // TRIGENB
}


void  Send_USB(byte data)
{
	while(input(TXE)) ;
	set_tris_D(0x00);
	port_D = data;	
	output_low(REWR);
	delay_us(1);
	output_high(REWR);
	set_tris_D(0xff);
}

void  Send_USB_Core(byte data)
{
	port_D = data;
	output_low(REWR);
	delay_cycles(1);
	output_high(REWR);
}

void PushReg(char c) {
	if(c == '0') {
		output_low(PIN_C7);
		output_high(PIN_C6);
		output_low(PIN_C6);
	} else {
		output_high(PIN_C7);
		output_high(PIN_C6);
		output_low(PIN_C6);
	}
}

byte bgetc() {
	char c;

	while(!bkbhit) ; 
	c = RX_Buff[next_out];
	next_out = (next_out+1) % BUFFER_SIZE;
	return(c);
}

void  Command_Dispat(void)
{
	byte  i,cmd[BUFFER_SIZE];
	int16 k;

	for(i=0;i<BUFFER_SIZE;i++) {
		cmd[i] = bgetc();
		if(cmd[i] == 0x0d) break;
	}

	for(i=0;i<BUFFER_SIZE;i++) {
		if(cmd[i] > ' ') break;	 // Skip LF etc
	}

	switch(cmd[i]) {
		case 'R':
			if(cmd[i+1] == 'S' && cmd[i+2] == 'T') { // Reset 
				output_low(PIN_C0);
				delay_us(4);
				output_high(PIN_C0);
			}

			else if(cmd[i+1] == 'E' && cmd[i+2] == 'Q' && cmd[i+3] == 'P' && cmd[i+4] == 'N') { // Product Name Request 
				Send_USB('A');
				Send_USB('C');
				Send_USB('K');
				Send_USB('P');
				Send_USB('N');
				Send_USB('U');
				Send_USB('S');
				Send_USB('B');
				Send_USB('L');
				Send_USB('O');
				Send_USB('G');
				Send_USB('V');
				Send_USB('0');
				Send_USB('1'); // version 1!!!
				Send_USB(0x0d);
			}

			else if(cmd[i+1] == 'E' && cmd[i+2] == 'Q' && cmd[i+3] == 'P' && cmd[i+4] == 'K') { // Paket Request 
				while(input(TXE)) ;
				set_tris_D(0x00);
				output_high(PIN_C3); // set PACKET high
				delay_us(1);

				for(k = 0; k < 256; k++) { // STROBE high/low 256 times 
					output_high(PIN_C4); // set STROBE high
					delay_cycles(1);
					Send_USB_Core(input_b());
					output_low(PIN_C4);	// set STROBE low
					delay_cycles(1);
				}

				output_low(PIN_C3); // set PACKET low 
				set_tris_D(0xff);
			}
	
			else if(cmd[i+1] == 'E' && cmd[i+2] == 'Q' && cmd[i+3] == 'T' && cmd[i+4] == 'R') { // Trig Request
				output_high(PIN_A1); // set TRIGENB high
				delay_cycles(20);
				if(input(PIN_A0) == 0) { 
					Send_USB('A');
					Send_USB('C');
					Send_USB('K');
					Send_USB('T');
					Send_USB('R');
					Send_USB('N'); // Trigger not found
					Send_USB(0x0d);
				} else {
					Send_USB('A');
					Send_USB('C');
					Send_USB('K');
					Send_USB('T');
					Send_USB('R');
					Send_USB('Y'); // Trigger found
					Send_USB(0x0d);
				}
			}
		break;

		case 'D':
			if(cmd[i+1] == 'L' && cmd[i+2] == 'D') { // Download 
				output_high(PIN_C2); // set DOWNLOAD high -- this is called before transfer
				delay_cycles(20);
			}
		break;

		case 'C':	  
			if(cmd[i+1] == 'A' && cmd[i+2] == 'P') { // Capture
				output_low(PIN_A1); // set TRIGENB low
				delay_cycles(20);
				output_low(PIN_C2); // set DOWNLOAD low -- this is called after transfer 
				output_high(PIN_C1); // set CAPTURE high
				delay_us(4);
				output_low(PIN_C1); // set CAPTURE low
			}
		break;

		case 'S':	 
			if(cmd[i+1] == 'D') { // Inject Serial Data 
				for(k = 2; k < 26; k++)	PushReg(cmd[i+k]);
				output_low(PIN_C7);
				// latch
				output_high(PIN_C5);
				output_low(PIN_C5);
			}
		break;

		default:
		break;
	}
	cr_flag = 0;
}



void main() {
	byte  tmpbyte;

//	setup_adc_ports(RA0_RA1_RA3_ANALOG);
//	setup_adc(ADC_CLOCK_INTERNAL);
//	setup_counters(RTCC_INTERNAL,RTCC_DIV_16);
//  commented out on 2006/09/14 since these three lines are not necessary

	Init_IO();

	output_low(PIN_A4);
	delay_ms(200);
	output_high(PIN_A4);
	delay_ms(200);
	output_low(PIN_A4);
	delay_ms(200);
	output_high(PIN_A4);
	delay_ms(200);
	output_low(PIN_A4);
	delay_ms(200);
	output_high(PIN_A4);

	while(TRUE) {
		if(!input(RXF)) { // FIFO can be read
			output_low(RERD);
			tmpbyte = port_D;
			output_high(RERD);
			if(tmpbyte == 0x0d) cr_flag = 1; // 0d -> '\r' -> Command_Dispat

			RX_Buff[next_in] = tmpbyte; // tmpbyte goes like 'C', 'A', 'P', '\r'
			tmpbyte = next_in;
			next_in = (next_in+1) % BUFFER_SIZE;
			if(next_in == next_out) next_in = tmpbyte; // Buffer full !!
		}
	
		if(cr_flag == 1) {
			Command_Dispat();
		}
	}
}

